


from apscheduler.schedulers.blocking import BlockingScheduler
from tablestore import *
from BaseDataMaintenance.dataSource.source import getConnect_ots,getAuth,is_internal
from BaseDataMaintenance.dataSource.interface import *
from multiprocessing import Queue as PQueue,Process
from BaseDataMaintenance.model.ots.document_product import *
from BaseDataMaintenance.model.ots.attachment import *
from BaseDataMaintenance.common.Utils import *
from BaseDataMaintenance.common.ossUtils import *
from BaseDataMaintenance.maintenance.product.htmlparser import *
import oss2
from BaseDataMaintenance.common.multiThread import MultiThreadHandler

parameter_status_no_bidfile = -1
parameter_status_to_process = 0
parameter_status_process_succeed = 1
parameter_status_process_failed = 2
parameter_status_process_jump = 3
parameter_status_not_found = 4

class Product_Attachment_Processor():

    def __init__(self,):
        self.ots_client = getConnect_ots()
        self.product_attachment_queue = PQueue()
        self.product_attachment_queue_size = 100
        self.set_product_attachment = set()
        self.attachment_hub_url = "https://attachment-hub.oss-cn-hangzhou.aliyuncs.com/"
        self.auth = getAuth()
        oss2.defaults.connection_pool_size = 100
        oss2.defaults.multiget_num_threads = 20
        if is_internal:
            self.bucket_url = "http://oss-cn-hangzhou-internal.aliyuncs.com"
        else:
            self.bucket_url = "http://oss-cn-hangzhou.aliyuncs.com"
        log("bucket_url:%s"%(self.bucket_url))
        self.attachment_bucket_name = "attachment-hub"
        self.bucket = oss2.Bucket(self.auth,self.bucket_url,self.attachment_bucket_name)
        self.current_path = os.path.dirname(__file__)
        self.download_path = "%s/%s"%(self.current_path,"download")

    def process_parameters_producer(self,):

        if self.product_attachment_queue.qsize()>self.product_attachment_queue_size/3:
            return
        bool_query = BoolQuery(must_queries=[
            TermQuery("parameter_status",parameter_status_to_process)
        ])
        list_id = []
        rows,next_token,total_count,is_all_succeed = self.ots_client.search(Document_product_table_name,Document_product_table_name+"_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("parameter_status")]),limit=100,get_total_count=True),
                                                                            ColumnsToGet([DOCUMENT_PRODUCT_BID_FILEMD5S,DOCUMENT_PRODUCT_NAME,DOCUMENT_PRODUCT_ORIGINAL_NAME],return_type=ColumnReturnType.SPECIFIED))

        list_data = getRow_ots(rows)
        for data in list_data:
            _id = data.get(DOCUMENT_PRODUCT_ID)
            if _id in self.set_product_attachment:
                continue
            self.product_attachment_queue.put(data)
            list_id.append(_id)
        while next_token:
            if self.product_attachment_queue.qsize()>=self.product_attachment_queue_size:
                break
            rows,next_token,total_count,is_all_succeed = self.ots_client.search(Document_product_table_name,Document_product_table_name+"_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                ColumnsToGet([DOCUMENT_PRODUCT_BID_FILEMD5S,DOCUMENT_PRODUCT_NAME,DOCUMENT_PRODUCT_ORIGINAL_NAME],return_type=ColumnReturnType.SPECIFIED))
            list_data = getRow_ots(rows)
            for data in list_data:
                _id = data.get(DOCUMENT_PRODUCT_ID)
                if _id in self.set_product_attachment:
                    continue
                self.product_attachment_queue.put(data)
                list_id.append(_id)
        self.set_product_attachment =  set(list_id)

    def process_parameters_handler(self,item,result_queue):
        bid_filemd5s = item.get(DOCUMENT_PRODUCT_BID_FILEMD5S)
        product_name = item.get(DOCUMENT_PRODUCT_NAME)
        product_original_name = item.get(DOCUMENT_PRODUCT_ORIGINAL_NAME)
        list_product = []
        if product_name is not None:
            list_product.append(product_name)
        if product_original_name is not None:
            list_product.extend(product_original_name.split("_"))
        list_product = list(set(list_product))
        dp = Document_product(item)
        if bid_filemd5s is None or bid_filemd5s=="" or len(list_product)==0:
            dp.setValue(DOCUMENT_PRODUCT_PARAMETER_STATUS,parameter_status_no_bidfile)
            dp.update_row(self.ots_client)
            return
        list_filemd5 = bid_filemd5s.split(",")
        _find = False
        _success = False
        for _filemd5 in list_filemd5:
            if _find:
                break
            atta = attachment({attachment_filemd5:_filemd5})
            if atta.fix_columns(self.ots_client,[attachment_path,attachment_filetype],True):
                objectPath = atta.getProperties().get(attachment_path)
                _filetype = atta.getProperties().get(attachment_filetype)
                if _filetype in ("doc","xls"):
                    if len(list_filemd5)==1:
                        dp.setValue(DOCUMENT_PRODUCT_PARAMETER_STATUS,parameter_status_process_jump,True)
                        dp.update_row(self.ots_client)
                        return
                    else:
                        continue
                localpath = "%s/%s.%s"%(self.download_path,_filemd5,_filetype)
                localhtml = "%s/%s.%s"%(self.download_path,_filemd5,"html")
                download_succeed = False
                try:
                    if not os.path.exists(localpath):
                        download_succeed = downloadFile(self.bucket,objectPath,localpath)
                    else:
                        download_succeed = True
                except Exception as e:
                    download_succeed = False
                if download_succeed:
                    try:
                        _html = ""
                        if os.path.exists(localhtml):
                            _html = open(localhtml,"r",encoding="utf8").read()
                            _success = True
                        if len(_html)>10:
                            _success = True
                        else:
                            _data_base64 = base64.b64encode(open(localpath,"rb").read())
                            _success,_html,swf_images,classification = getAttachDealInterface(_data_base64,_filetype,url="http://192.168.2.102:15011/convert",kwargs={'page_no': '1,-1',"max_bytes":"-1"},timeout=6000)
                            if _success:
                                localhtml = "%s/%s.%s"%(self.download_path,_filemd5,"html")
                                with open(localhtml,"w",encoding="utf8") as f:
                                    f.write(_html)
                        if _success:
                            if len(_html)>5:
                                pd = ParseDocument(_html,True)

                                list_text = []
                                for _product in list_product:
                                    pd.fix_tree(_product)
                                    list_data = pd.tree
                                    _text,_count = extract_product_parameters(list_data,_product)
                                    if _count>0:
                                        _find = True
                                    if _text is not None:
                                        list_text.append(_text)
                                pd = ParseDocument(_html,False)

                                list_text = []
                                for _product in list_product:
                                    pd.fix_tree(_product)
                                    list_data = pd.tree
                                    _text,_count = extract_product_parameters(list_data,_product)
                                    if _count>0:
                                        _find = True
                                    if _text is not None:
                                        list_text.append(_text)
                                if len(list_text)>0:
                                    list_text.sort(key=lambda x:len(re.findall('[:：;；]',BeautifulSoup(x,"html5lib").get_text())), reverse=True)
                                    _text = list_text[0]
                                    _success = True
                                    dp.setValue(DOCUMENT_PRODUCT_PARAMETER,_text,True)
                                    dp.setValue(DOCUMENT_PRODUCT_PARAMETER_STATUS,parameter_status_process_succeed,True)
                                    dp.update_row(self.ots_client)
                                    return
                            else:
                                log("product attachment process filemd5 %s has no content"%(_filemd5))
                    except Exception as e:
                        traceback.print_exc()
                    finally:
                        try:
                            # if os.path.exists(localpath):
                            #     os.remove(localpath)
                            pass
                        except Exception as e:
                            pass

        if not _find:
            dp.setValue(DOCUMENT_PRODUCT_PARAMETER_STATUS,parameter_status_not_found,True)
            dp.update_row(self.ots_client)
        else:
            dp.setValue(DOCUMENT_PRODUCT_PARAMETER_STATUS,parameter_status_process_failed,True)
            dp.update_row(self.ots_client)

    def start_process(self):
        mt = MultiThreadHandler(self.product_attachment_queue,self.process_parameters_handler,None,3,need_stop=False,restart=True)
        mt.run()

    def process_parameters_comsumer(self,):

        # process_count = 2
        # list_process = []
        # for i in range(process_count):
        #     p = Process(target=self.start_process)
        #     list_process.append(p)
        # for p in list_process:
        #     p.start()
        # for p in list_process:
        #     p.join()

        self.start_process()

    def start_process_parameters(self):
        scheduler = BlockingScheduler()
        scheduler.add_job(self.process_parameters_producer,"cron",second="*/10")
        scheduler.add_job(self.process_parameters_comsumer,"cron",second="*/30")
        scheduler.start()

def start_process_parameters():
    pap = Product_Attachment_Processor()
    pap.start_process_parameters()

def change_parameters_status():
    ots_client =getConnect_ots()
    bool_query = BoolQuery(must_queries=[
        RangeQuery("parameter_status",-1)
    ],
                           must_not_queries=[
        TermQuery("parameter_status",parameter_status_to_process),
        TermQuery("parameter_status",parameter_status_process_succeed),
        TermQuery("parameter_status",parameter_status_process_jump),
        # TermQuery("parameter_status",parameter_status_no_bidfile),

    ])
    list_data = []
    rows,next_token,total_count,is_all_succeed = ots_client.search(Document_product_table_name,Document_product_table_name+"_index",
                                                                        SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("parameter_status")]),limit=100,get_total_count=True),
                                                                        ColumnsToGet([DOCUMENT_PRODUCT_BID_FILEMD5S,DOCUMENT_PRODUCT_NAME,DOCUMENT_PRODUCT_ORIGINAL_NAME],return_type=ColumnReturnType.SPECIFIED))

    list_data.extend(getRow_ots(rows))
    print("total_count",total_count)
    while next_token:
        rows,next_token,total_count,is_all_succeed = ots_client.search(Document_product_table_name,Document_product_table_name+"_index",
                                                                            SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                            ColumnsToGet([DOCUMENT_PRODUCT_BID_FILEMD5S],return_type=ColumnReturnType.SPECIFIED))
        list_data.extend(getRow_ots(rows))
    for data in list_data:
        dp = Document_product(data)
        dp.setValue(DOCUMENT_PRODUCT_PARAMETER_STATUS,parameter_status_to_process,True)
        dp.setValue(DOCUMENT_PRODUCT_PARAMETER,"",True)
        dp.update_row(ots_client)

if __name__ == '__main__':
    start_process_parameters()
    # change_parameters_status()